@@ -17,10 +17,6 @@
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif // __BORLANDC__
-
 #ifndef WX_PRECOMP
     #include "wx/wx.h"
 #endif // WX_PRECOMP
@@ -40,16 +36,16 @@
 // Global data
 // ----------------------------------------------------------------------------
 
-MyData g_data;
-
 wxString g_listbox_choices[] =
-    {wxT("one"),  wxT("two"),  wxT("three")};
+    {"one",  "two",  "three"};
 
 wxString g_combobox_choices[] =
-    {wxT("yes"), wxT("no (doesn't validate)"), wxT("maybe (doesn't validate)")};
+    {"yes", "no (doesn't validate)", "maybe (doesn't validate)"};
 
 wxString g_radiobox_choices[] =
-    {wxT("green"), wxT("yellow"), wxT("red")};
+    {"green", "yellow", "red"};
+
+MyData g_data;
 
 // ----------------------------------------------------------------------------
 // MyData
@@ -60,9 +56,10 @@ MyData::MyData()
     // This string will be passed to an alpha-only validator, which
     // will complain because spaces aren't alpha. Note that validation
     // is performed only when 'OK' is pressed.
-    m_string = wxT("Spaces are invalid here");
+    m_string = "Spaces are invalid here";
     m_string2 = "Valid text";
     m_listbox_choices.Add(0);
+    m_combobox_choice = g_combobox_choices[0];
     m_intValue = 0;
     m_smallIntValue = 3;
     m_doubleValue = 12354.31;
@@ -137,7 +134,7 @@ bool MyApp::OnInit()
         return false;
 
     // Create and display the main frame window.
-    MyFrame *frame = new MyFrame((wxFrame *) NULL, wxT("Validator Test"),
+    MyFrame *frame = new MyFrame((wxFrame *) NULL, "Validator Test",
                                  50, 50, 300, 250);
     frame->Show(true);
 
@@ -162,17 +159,17 @@ MyFrame::MyFrame(wxFrame *frame, const wxString&title, int x, int y, int w, int
 
     // Create a listbox to display the validated data.
     m_listbox = new wxListBox(this, wxID_ANY);
-    m_listbox->Append(wxString(wxT("Try 'File|Test' to see how validators work.")));
+    m_listbox->Append(wxString("Try 'File|Test' to see how validators work."));
 
     wxMenu *file_menu = new wxMenu;
 
-    file_menu->Append(VALIDATE_TEST_DIALOG, wxT("&Test dialog...\tCtrl-T"), wxT("Demonstrate validators"));
-    file_menu->AppendCheckItem(VALIDATE_TOGGLE_BELL, wxT("&Bell on error"), wxT("Toggle bell on error"));
+    file_menu->Append(VALIDATE_TEST_DIALOG, "&Test dialog...\tCtrl-T", "Demonstrate validators");
+    file_menu->AppendCheckItem(VALIDATE_TOGGLE_BELL, "&Bell on error", "Toggle bell on error");
     file_menu->AppendSeparator();
-    file_menu->Append(wxID_EXIT, wxT("E&xit"));
+    file_menu->Append(wxID_EXIT, "E&xit");
 
     wxMenuBar *menu_bar = new wxMenuBar;
-    menu_bar->Append(file_menu, wxT("&File"));
+    menu_bar->Append(file_menu, "&File");
     SetMenuBar(menu_bar);
 
     // All validators share a common (static) flag that controls
@@ -196,7 +193,7 @@ void MyFrame::OnTestDialog(wxCommandEvent& WXUNUSED(event))
     // and variables together. Values are transferred between them behind
     // the scenes, so here we don't have to query the controls for their
     // values.
-    MyDialog dialog(this, wxT("Validator demonstration"));
+    MyDialog dialog(this, "Validator demonstration");
 
     // When the dialog is displayed, validators automatically transfer
     // data from variables to their corresponding controls.
@@ -206,24 +203,24 @@ void MyFrame::OnTestDialog(wxCommandEvent& WXUNUSED(event))
         // automatically transferred to the variables we specified
         // when we created the validators.
         m_listbox->Clear();
-        m_listbox->Append(wxString(wxT("string: ")) + g_data.m_string);
-        m_listbox->Append(wxString(wxT("string #2: ")) + g_data.m_string2);
+        m_listbox->Append(wxString("string: ") + g_data.m_string);
+        m_listbox->Append(wxString("string #2: ") + g_data.m_string2);
 
         for(unsigned int i = 0; i < g_data.m_listbox_choices.GetCount(); ++i)
         {
             int j = g_data.m_listbox_choices[i];
-            m_listbox->Append(wxString(wxT("listbox choice(s): ")) + g_listbox_choices[j]);
+            m_listbox->Append(wxString("listbox choice(s): ") + g_listbox_choices[j]);
         }
 
-        wxString checkbox_state(g_data.m_checkbox_state ? wxT("checked") : wxT("unchecked"));
-        m_listbox->Append(wxString(wxT("checkbox: ")) + checkbox_state);
-        m_listbox->Append(wxString(wxT("combobox: ")) + g_data.m_combobox_choice);
-        m_listbox->Append(wxString(wxT("radiobox: ")) + g_radiobox_choices[g_data.m_radiobox_choice]);
+        wxString checkbox_state(g_data.m_checkbox_state ? "checked" : "unchecked");
+        m_listbox->Append(wxString("checkbox: ") + checkbox_state);
+        m_listbox->Append(wxString("combobox: ") + g_data.m_combobox_choice);
+        m_listbox->Append(wxString("radiobox: ") + g_radiobox_choices[g_data.m_radiobox_choice]);
 
         m_listbox->Append(wxString::Format("integer value: %d", g_data.m_intValue));
         m_listbox->Append(wxString::Format("small int value: %u", g_data.m_smallIntValue));
         m_listbox->Append(wxString::Format("double value: %.3f", g_data.m_doubleValue));
-        m_listbox->Append(wxString::Format("percent value: %.4f", g_data.m_percentValue));
+        m_listbox->Append(wxString::Format("percent value: %.4f", double(g_data.m_percentValue)));
     }
 }
 
@@ -247,28 +244,19 @@ MyDialog::MyDialog( wxWindow *parent, const wxString& title,
 
     wxFlexGridSizer *flexgridsizer = new wxFlexGridSizer(3, 2, 5, 5);
 
-    // Create and add controls to sizers. Note that a member variable
-    // of g_data is bound to each control upon construction. There is
-    // currently no easy way to substitute a different validator or a
-    // different transfer variable after a control has been constructed.
-
+    // Create and add controls to sizers.
     // Pointers to some of these controls are saved in member variables
     // so that we can use them elsewhere, like this one.
-    m_text = new wxTextCtrl(this, VALIDATE_TEXT, wxEmptyString,
-                            wxDefaultPosition, wxDefaultSize, 0,
-                            wxTextValidator(wxFILTER_ALPHA, &g_data.m_string));
-    m_text->SetToolTip("uses wxTextValidator with wxFILTER_ALPHA");
+    m_text = new wxTextCtrl(this, VALIDATE_TEXT);
+    m_text->SetToolTip("wxTextValidator not set");
+    m_text->SetHint("Enter some text here, please...");
     flexgridsizer->Add(m_text, 1, wxGROW);
 
-
-    // Now set a wxTextValidator with an explicit list of characters NOT allowed:
-    wxTextValidator textVal(wxFILTER_EMPTY|wxFILTER_EXCLUDE_CHAR_LIST, &g_data.m_string2);
-    textVal.SetCharExcludes("bcwyz");
-    wxTextCtrl* txt2 =
-             new wxTextCtrl(this, VALIDATE_TEXT2, wxEmptyString,
-                            wxDefaultPosition, wxDefaultSize, 0, textVal);
-    txt2->SetToolTip("uses wxTextValidator with wxFILTER_EMPTY|wxFILTER_EXCLUDE_CHAR_LIST to exclude 'bcwyz'");
-    flexgridsizer->Add(txt2, 1, wxGROW);
+    // Make it possible to change the wxTextValidator for m_text at runtime.
+    wxButton* const button =
+        new wxButton(this, wxID_ANY, "Set new wxTextValidator...");
+    button->Bind(wxEVT_BUTTON, &MyDialog::OnChangeValidator, this);
+    flexgridsizer->Add(button, wxSizerFlags().Center());
 
     flexgridsizer->Add(new wxListBox((wxWindow*)this, VALIDATE_LIST,
                         wxDefaultPosition, wxDefaultSize,
@@ -287,7 +275,7 @@ MyDialog::MyDialog( wxWindow *parent, const wxString& title,
     // because we don't use it elsewhere--it can be anonymous.
     // We don't need any such pointer to query its state, which
     // can be gotten directly from g_data.
-    flexgridsizer->Add(new wxCheckBox(this, VALIDATE_CHECK, wxT("Sample checkbox"),
+    flexgridsizer->Add(new wxCheckBox(this, VALIDATE_CHECK, "Sample checkbox",
                         wxDefaultPosition, wxDefaultSize, 0,
                         wxGenericValidator(&g_data.m_checkbox_state)),
                        1, wxALIGN_CENTER|wxALL, 15);
@@ -375,7 +363,7 @@ MyDialog::MyDialog( wxWindow *parent, const wxString& title,
 
     mainsizer->Add(flexgridsizer, 1, wxGROW | wxALL, 10);
 
-    mainsizer->Add(new wxRadioBox((wxWindow*)this, VALIDATE_RADIO, wxT("Pick a color"),
+    mainsizer->Add(new wxRadioBox((wxWindow*)this, VALIDATE_RADIO, "Pick a color",
                                     wxDefaultPosition, wxDefaultSize,
                                     3, g_radiobox_choices, 1, wxRA_SPECIFY_ROWS,
                                     wxGenericValidator(&g_data.m_radiobox_choice)),
@@ -391,17 +379,298 @@ MyDialog::MyDialog( wxWindow *parent, const wxString& title,
 
     // make the dialog a bit bigger than its minimal size:
     SetSize(GetBestSize()*1.5);
-}
 
-bool MyDialog::TransferDataToWindow()
-{
-    bool r = wxDialog::TransferDataToWindow();
-
-    // These function calls have to be made here, after the
-    // dialog has been created.
+    // Now sets the focus to m_text
     m_text->SetFocus();
-    m_combobox->SetSelection(0);
-
-    return r;
 }
 
+void MyDialog::OnChangeValidator(wxCommandEvent& WXUNUSED(event))
+{
+    TextValidatorDialog dialog(this, m_text);
+
+    if ( dialog.ShowModal() == wxID_OK )
+    {
+        dialog.ApplyValidator();
+    }
+}
+
+// ----------------------------------------------------------------------------
+// TextValidatorDialog
+// ----------------------------------------------------------------------------
+
+TextValidatorDialog::TextValidatorDialog(wxWindow *parent, wxTextCtrl* txtCtrl)
+    : wxDialog(parent, wxID_ANY, "wxTextValidator Dialog"),
+      m_txtCtrl(txtCtrl),
+      m_noValidation(true),
+      m_validatorStyle(wxFILTER_NONE)
+{
+    if ( m_txtCtrl )
+    {
+        wxTextValidator* txtValidator =
+            wxDynamicCast(m_txtCtrl->GetValidator(), wxTextValidator);
+
+        if ( txtValidator )
+        {
+            m_validatorStyle = txtValidator->GetStyle();
+
+            if ( m_validatorStyle != wxFILTER_NONE )
+                m_noValidation = false;
+
+            m_charIncludes = txtValidator->GetCharIncludes();
+            m_charExcludes = txtValidator->GetCharExcludes();
+            m_includes = txtValidator->GetIncludes();
+            m_excludes = txtValidator->GetExcludes();
+        }
+    }
+
+    wxFlexGridSizer *fgSizer = new wxFlexGridSizer(2, FromDIP(wxSize(5, 5)));
+    const wxSizerFlags center = wxSizerFlags().CenterVertical();
+
+    const StyleValidator styleVal(&m_validatorStyle);
+
+    wxCheckBox* filterNone = new wxCheckBox(this, Id_None, "wxFILTER_NONE");
+    filterNone->SetValue(m_noValidation);
+    fgSizer->Add(filterNone);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "No filtering takes place."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_Empty, "wxFILTER_EMPTY"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Empty strings are filtered out."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_Ascii, "wxFILTER_ASCII"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Non-ASCII characters are filtered out."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_Alpha, "wxFILTER_ALPHA"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Non-alpha characters are filtered out."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_Alphanumeric, "wxFILTER_ALPHANUMERIC"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Non-alphanumeric characters are filtered out."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_Digits, "wxFILTER_DIGITS"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Non-digit characters are filtered out."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_Numeric, "wxFILTER_NUMERIC"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Non-numeric characters are filtered out."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_IncludeList, "wxFILTER_INCLUDE_LIST"), center)
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxTextCtrl(this, Id_IncludeListTxt), wxSizerFlags().Expand())
+        ->GetWindow()->Bind(wxEVT_KILL_FOCUS, &TextValidatorDialog::OnKillFocus, this);
+
+    fgSizer->Add(new wxCheckBox(this, Id_IncludeCharList, "wxFILTER_INCLUDE_CHAR_LIST"), center)
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxTextCtrl(this, Id_IncludeCharListTxt, wxString(), wxDefaultPosition,
+                                wxDefaultSize, 0, wxGenericValidator(&m_charIncludes)),
+                 wxSizerFlags().Expand())
+        ->GetWindow()->Bind(wxEVT_KILL_FOCUS, &TextValidatorDialog::OnKillFocus, this);
+
+    fgSizer->Add(new wxCheckBox(this, Id_ExcludeList, "wxFILTER_EXCLUDE_LIST"), center)
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxTextCtrl(this, Id_ExcludeListTxt), wxSizerFlags().Expand())
+        ->GetWindow()->Bind(wxEVT_KILL_FOCUS, &TextValidatorDialog::OnKillFocus, this);
+
+    fgSizer->Add(new wxCheckBox(this, Id_ExcludeCharList, "wxFILTER_EXCLUDE_CHAR_LIST"), center)
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxTextCtrl(this, Id_ExcludeCharListTxt, wxString(), wxDefaultPosition,
+                                wxDefaultSize, 0, wxGenericValidator(&m_charExcludes)),
+                 wxSizerFlags().Expand())
+        ->GetWindow()->Bind(wxEVT_KILL_FOCUS, &TextValidatorDialog::OnKillFocus, this);
+
+    fgSizer->Add(new wxCheckBox(this, Id_Xdigits, "wxFILTER_XDIGITS"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Non-xdigit characters are filtered out."));
+
+    fgSizer->Add(new wxCheckBox(this, Id_Space, "wxFILTER_SPACE"))
+        ->GetWindow()->SetValidator(styleVal);
+    fgSizer->Add(new wxStaticText(this, wxID_ANY, "Allow spaces."));
+
+    // Set the main sizer.
+    wxBoxSizer *mainsizer = new wxBoxSizer( wxVERTICAL );
+
+    mainsizer->Add(fgSizer, wxSizerFlags(1).Border(wxALL, 10).Expand());
+
+    mainsizer->Add(CreateButtonSizer(wxOK | wxCANCEL),
+                   wxSizerFlags().Expand().DoubleBorder());
+
+    SetSizer(mainsizer);
+    mainsizer->SetSizeHints(this);
+
+    // Bind event handlers.
+    Bind(wxEVT_CHECKBOX, &TextValidatorDialog::OnChecked, this);
+    Bind(wxEVT_UPDATE_UI, &TextValidatorDialog::OnUpdateUI,
+         this, Id_Ascii, Id_ExcludeCharListTxt);
+}
+
+void TextValidatorDialog::OnUpdateUI(wxUpdateUIEvent& event)
+{
+    event.Enable(!m_noValidation);
+
+    if ( m_noValidation )
+        event.Check(false);
+}
+
+void TextValidatorDialog::OnChecked(wxCommandEvent& event)
+{
+    if ( event.GetId() == Id_None )
+    {
+        m_noValidation = event.IsChecked();
+
+        if ( m_noValidation )
+        {
+            long style = wxFILTER_NONE;
+
+            // we should keep this flag on if it has been set.
+            if ( HasFlag(wxFILTER_EMPTY) )
+                style = wxFILTER_EMPTY;
+
+            m_validatorStyle = style;
+
+            m_charIncludes.clear();
+            m_charExcludes.clear();
+            m_includes.clear();
+            m_excludes.clear();
+        }
+    }
+}
+
+void TextValidatorDialog::OnKillFocus(wxFocusEvent &event)
+{
+    wxTextCtrl* txtCtrl = wxDynamicCast(event.GetEventObject(), wxTextCtrl);
+
+    if ( txtCtrl && txtCtrl->IsModified() )
+    {
+        const int id = event.GetId();
+
+        if ( id == Id_IncludeCharListTxt )
+        {
+            m_charIncludes = txtCtrl->GetValue();
+        }
+        else if ( id == Id_ExcludeCharListTxt )
+        {
+            m_charExcludes = txtCtrl->GetValue();
+        }
+        else if ( id == Id_IncludeListTxt )
+        {
+            m_includes = wxSplit(txtCtrl->GetValue(), ' ');
+        }
+        else if ( id == Id_ExcludeListTxt )
+        {
+            m_excludes = wxSplit(txtCtrl->GetValue(), ' ');
+        }
+    }
+
+    event.Skip();
+}
+
+void TextValidatorDialog::ApplyValidator()
+{
+    if ( !m_txtCtrl )
+        return;
+
+    wxString tooltip = "uses wxTextValidator with ";
+
+    if ( m_noValidation )
+    {
+        tooltip += "wxFILTER_NONE|";
+    }
+    else
+    {
+        if ( HasFlag(wxFILTER_ASCII) )
+            tooltip += "wxFILTER_ASCII|";
+        if ( HasFlag(wxFILTER_ALPHA) )
+            tooltip += "wxFILTER_ALPHA|";
+        if ( HasFlag(wxFILTER_ALPHANUMERIC) )
+            tooltip += "wxFILTER_ALPHANUMERIC|";
+        if ( HasFlag(wxFILTER_DIGITS) )
+            tooltip += "wxFILTER_DIGITS|";
+        if ( HasFlag(wxFILTER_NUMERIC) )
+            tooltip += "wxFILTER_NUMERIC|";
+        if ( HasFlag(wxFILTER_XDIGITS) )
+            tooltip += "wxFILTER_XDIGITS|";
+        if ( HasFlag(wxFILTER_SPACE) )
+            tooltip += "wxFILTER_SPACE|";
+        if ( HasFlag(wxFILTER_INCLUDE_LIST) )
+            tooltip += "wxFILTER_INCLUDE_LIST|";
+        if ( HasFlag(wxFILTER_INCLUDE_CHAR_LIST) )
+            tooltip += "wxFILTER_INCLUDE_CHAR_LIST|";
+        if ( HasFlag(wxFILTER_EXCLUDE_LIST) )
+            tooltip += "wxFILTER_EXCLUDE_LIST|";
+        if ( HasFlag(wxFILTER_EXCLUDE_CHAR_LIST) )
+            tooltip += "wxFILTER_EXCLUDE_CHAR_LIST|";
+    }
+
+    if ( HasFlag(wxFILTER_EMPTY) )
+    {
+        tooltip += "wxFILTER_EMPTY|";
+    }
+
+    tooltip.RemoveLast(); // remove the trailing '|' char.
+
+    if ( !m_charIncludes.empty() )
+    {
+        tooltip += "\nAllowed chars: ";
+        tooltip += m_charIncludes;
+    }
+
+    if ( !m_charExcludes.empty() )
+    {
+        tooltip += "\nDisallowed chars: ";
+        tooltip += m_charExcludes;
+    }
+
+    m_txtCtrl->SetToolTip(tooltip);
+
+    // Prepare and set the wxTextValidator
+    wxTextValidator txtVal(m_validatorStyle, &g_data.m_string);
+    txtVal.SetCharIncludes(m_charIncludes);
+    txtVal.SetCharExcludes(m_charExcludes);
+    txtVal.SetIncludes(m_includes);
+    txtVal.SetExcludes(m_excludes);
+
+    m_txtCtrl->SetValidator(txtVal);
+    m_txtCtrl->SetFocus();
+}
+
+bool TextValidatorDialog::StyleValidator::TransferToWindow()
+{
+    wxASSERT( wxDynamicCast(m_validatorWindow, wxCheckBox) );
+
+    if ( m_style )
+    {
+        wxCheckBox* cb = (wxCheckBox*)GetWindow();
+        if ( !cb )
+            return false;
+
+        const long style = 1L << (cb->GetId()-wxID_HIGHEST-1);
+
+        cb->SetValue((*m_style & style) != 0);
+    }
+
+    return true;
+}
+
+bool TextValidatorDialog::StyleValidator::TransferFromWindow()
+{
+    wxASSERT( wxDynamicCast(m_validatorWindow, wxCheckBox) );
+
+    if ( m_style )
+    {
+        wxCheckBox* cb = (wxCheckBox*)GetWindow();
+        if ( !cb )
+            return false;
+
+        const long style = 1L << (cb->GetId()-wxID_HIGHEST-1);
+
+        if ( cb->IsChecked() )
+            *m_style |= style;
+        else
+            *m_style &= ~style;
+    }
+
+    return true;
+}
